/**
 * @file rdi_tunnel.h
 * @brief The definitions and prototypes for the RDI
 *        tunneling application.
 * 
 * @copyright Copyright (c) 2024
 */
#ifndef RDI_TUNNEL_H
#define RDI_TUNNEL_H

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

#define LOCAL_PORT_OPTION       'a'
#define TETHER_PORT_OPTION      'b'
#define TETHER_HOST_OPTION      'c'
#define LOG_OPTION              'd'
#define HELP_OPTION             'e'
#define VERSION_OPTION          'f'
#define LOCAL_RETRY_OPTION      'g'

#define PATH_SEPARATOR          '/'
#define DEFAULT_RETRIES         10
#define DEFAULT_BUFFER_SIZE     (256 * 1024)

#define INVALID_SOCKET          -1

//---------------------------------------------------------------------------
// Structure definitions
//---------------------------------------------------------------------------
struct struct_settings
{
    unsigned int local_port     : 1;
    unsigned int tether_port    : 1;
    unsigned int tether_host    : 1;
    unsigned int log            : 1;
    unsigned int local_retry    : 1;
};

struct struct_options
{
    const char *local_port;
    const char *tether_host;
    const char *tether_port;
    int local_retry;
};

struct struct_rc
{
    int tether_socket;
    int local_socket;

    struct sockaddr_in tether_addr;
    struct sockaddr_in local_addr;
    struct hostent *tether_host;
};

//---------------------------------------------------------------------------
// Variable declarations
//---------------------------------------------------------------------------
const char *name;

//---------------------------------------------------------------------------
// Function prototypes
//---------------------------------------------------------------------------
bool build_tether();
bool wait_for_tether();
bool build_local();
void use_tunnel();
int fd();

void set_options(int argc, char *argv[]);

void print_help();
void print_helpinfo();
void print_usage();
void print_version();
void print_missing(const char *message);

#endif /* RDI_TUNNEL_H */
